local require = GLOBAL.require
STRINGS = GLOBAL.STRINGS
RECIPETABS = GLOBAL.RECIPETABS
Recipe = GLOBAL.Recipe
Ingredient = GLOBAL.Ingredient
TECH = GLOBAL.TECH
RECIPE_GAME_TYPE = GLOBAL.RECIPE_GAME_TYPE
GetWorld = GLOBAL.GetWorld
GetTime = GLOBAL.GetTime
GetPlayer = GLOBAL.GetPlayer
SpawnPrefab = GLOBAL.SpawnPrefab
SpawnSaveRecord = GLOBAL.SpawnSaveRecord
STRINGS = GLOBAL.STRINGS

STRINGS.RECIPE_DESC.ABIGAIL_FLOWER = "a magical keepsake."
local function getstatus(inst)
    if inst._chargestate == 3 then
        return inst.components.inventoryitem.owner ~= nil
            and "HAUNTED_POCKET"
            or "HAUNTED_GROUND"
    end
    local time_charge = inst.components.cooldown:GetTimeToCharged()
    return (time_charge < TUNING.TOTAL_DAY_TIME * .5 and "SOON")
        or (time_charge < TUNING.TOTAL_DAY_TIME * 2 and "MEDIUM")
        or "LONG"
end

local function activate(inst)
    inst.SoundEmitter:PlaySound("dontstarve/common/haunted_flower_LP", "loop")
    inst:DoTaskInTime(0.5, function()
        -- body
        inst.AnimState:PlayAnimation("haunted_pre")
        inst.AnimState:PushAnimation("idle_haunted_loop", true)
    end)

    inst:ListenForEvent("entity_death", inst._onentitydeath, GetWorld())
end

local function deactivate(inst)
    inst.SoundEmitter:KillAllSounds()
    inst:RemoveEventCallback("entity_death", inst._onentitydeath, GetWorld())
end

local function IsValidLink(inst, player)
    return player.abigail == nil
end

local function dodecay(inst)
    -- print("dodecay被执行")
    local x, y, z = inst.Transform:GetWorldPosition()
    local canplant =
        GetWorld().Map:CanPlantAtPoint(x, 0, z) and
        GetWorld().Map:CanPlacePrefabFilteredAtPoint(x, 0, z, "flower") and
        not (RoadManager ~= nil and RoadManager:IsOnRoad(x, 0, z))

    if canplant then
        local radius = inst:GetPhysicsRadius(0)
        for i, v in ipairs(TheSim:FindEntities(x, 0, z, 3, nil, { "NOBLOCK", "_inventoryitem", "locomotor", "FX", "INLIMBO", "DECOR" })) do
            if v ~= inst and v.entity:IsVisible() then
                local spacing = radius + v:GetPhysicsRadius(.25) - .001
                if v:GetDistanceSqToPoint(x, 0, z) < (v.deploy_extra_spacing ~= nil and math.max(v.deploy_extra_spacing * v.deploy_extra_spacing, spacing * spacing) or spacing * spacing) then
                    canplant = false
                    break
                end
            end
        end
    end

    if canplant then
        SpawnPrefab("planted_flower").Transform:SetPosition(x, 0, z)
    else
        inst:AddComponent("lootdropper")
        inst.components.lootdropper:SpawnLootPrefab("petals", Vector3(x, y, z))
    end

    if not inst:IsAsleep() then
        SpawnPrefab("small_puff").Transform:SetPosition(x, y, z)
    end

    inst:Remove()
end

local function startdecay(inst)
    -- print("startdecay",inst._decaytask == nil)
    if inst._decaytask == nil then
        -- print("startdecay")
        -- inst._decaytask = inst:DoTaskInTime(TUNING.TOTAL_DAY_TIME*3, dodecay)
        inst._decaytask = inst:DoTaskInTime(5, dodecay)

        inst._decaystart = GetTime()
    end
end

local function stopdecay(inst)
    if inst._decaytask ~= nil then
        inst._decaytask:Cancel()
        inst._decaytask = nil
        inst._decaystart = nil
    end
end

local function onsave(inst, data)
    -- if inst._decaystart ~= nil then
    --     local time = GetTime() - inst._decaystart
    --     if time > 0 then
    --         data.decaytime = time
    --     end
    -- end
end

local function onload(inst, data)
    if inst._decaytask ~= nil and data ~= nil and data.decaytime ~= nil then
        -- local remaining = math.max(0, TUNING.TOTAL_DAY_TIME*3 - data.decaytime)
        inst._decaytask:Cancel()
        -- inst._decaytask = inst:DoTaskInTime(remaining, dodecay)
        -- inst._decaystart = GetTime() + remaining - TUNING.TOTAL_DAY_TIME*3
    end
end

local function updatestate(inst)
	-- print("updatestate","是否为nil",inst._playerlink == nil)
    if inst._playerlink ~= nil and inst.components.cooldown:IsCharged() then
        if inst._chargestate ~= 3 then
            inst._chargestate = 3
            inst.components.inventoryitem:ChangeImageName("abigail_flower_haunted")
            inst.AnimState:PlayAnimation("haunted_pre")
            inst.AnimState:PushAnimation("idle_haunted_loop", true)
            inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
            if inst.components.inventoryitem.owner == nil then
                activate(inst)
            end
        end
    else
        if inst._chargestate == 3 then
            inst.AnimState:ClearBloomEffectHandle()
            deactivate(inst)
        end
        if inst._playerlink ~= nil and inst.components.cooldown:GetTimeToCharged() < TUNING.TOTAL_DAY_TIME then
            if inst._chargestate ~= 2 then
                inst._chargestate = 2
                inst.components.inventoryitem:ChangeImageName("abigail_flower2")
                inst.AnimState:PlayAnimation("idle_2")
            end
        elseif inst._chargestate ~= 1 then
            inst._chargestate = 1
            inst.components.inventoryitem:ChangeImageName("abigail_flower")
            inst.AnimState:PlayAnimation("idle_1")
        end
    end
end

local function ondeath(inst, deadthing, killer)
	-- print("ondeath被调用")
    if inst._chargestate == 3 and
        inst._playerlink ~= nil and
        inst._playerlink.abigail == nil and
        inst._playerlink.components.leader ~= nil and
        inst.components.inventoryitem.owner == nil and
        deadthing ~= nil and
        not (deadthing:HasTag("wall") or deadthing:HasTag("smashable")) then

        if deadthing:IsValid() then
            if not inst:IsNear(deadthing, 16) then
                return
            end
        elseif killer == nil or not inst:IsNear(killer, 16) then
            return
        end

        inst._playerlink.components.sanity:DoDelta(-TUNING.SANITY_HUGE)
        local abigail = SpawnPrefab("abigail")
        if abigail ~= nil then
            abigail.Transform:SetPosition(inst.Transform:GetWorldPosition())
            abigail.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
            abigail:LinkToPlayer(inst._playerlink)
            inst:Remove()
        end
    end
end

local function linktoplayer(inst, player)
    if player ~= nil and IsValidLink(inst, player) then
    	if player.abigail_flowers == nil then
    		player.abigail_flowers = {}
    	end
        inst:ListenForEvent("onremove", inst._onremoveplayer, player)
        inst:ListenForEvent("killed", inst._onplayerkillthing, player)
        inst._playerlink = player
        player.abigail_flowers[inst] = true
    end
end

local function unlink(inst)
    if inst._playerlink ~= nil then
        inst:RemoveEventCallback("onremove", inst._onremoveplayer, inst._playerlink)
        inst:RemoveEventCallback("killed", inst._onplayerkillthing, inst._playerlink)
        inst._playerlink.abigail_flowers[inst] = nil
        inst._playerlink = nil
    end
end

local function storeincontainer(inst, container)
    if container ~= nil and container.components.container ~= nil then
        inst:ListenForEvent("onopen", inst._ontogglecontainer, container)
        inst:ListenForEvent("onclose", inst._ontogglecontainer, container)
        inst:ListenForEvent("onremove", inst._onremovecontainer, container)
        inst._container = container
    end
end

local function unstore(inst)
    if inst._container ~= nil then
        inst:RemoveEventCallback("onopen", inst._ontogglecontainer, inst._container)
        inst:RemoveEventCallback("onclose", inst._ontogglecontainer, inst._container)
        inst:RemoveEventCallback("onremove", inst._onremovecontainer, inst._container)
        inst._container = nil
    end
end

local function topocket(inst, owner)
	owner = GetPlayer()
	-- print("topocket","owner",owner,"end")
    -- stopdecay(inst)
    deactivate(inst)
    if inst._container ~= owner then
        unstore(inst)
        storeincontainer(inst, owner)
    end
    if owner.components.container ~= nil then
        owner = owner.components.container.opener
    end
    if inst._playerlink ~= owner then
        unlink(inst)
        linktoplayer(inst, owner)
        updatestate(inst)
    end
end

local function toground(inst)
	-- print("togroud 被调用")
	-- print("_chargestate",inst._chargestate)
    unstore(inst)
    if inst._chargestate == 3 then
        activate(inst)
    elseif inst._playerlink == nil then
        -- startdecay(inst)
    end
end

local function refresh(inst)
    if inst._playerlink == nil then
        local owner = GetPlayer()
        if owner ~= nil then
            if owner.components.container ~= nil then
                owner = owner.components.container.opener
            end
            linktoplayer(inst, owner)
            updatestate(inst)
        end
    elseif not IsValidLink(inst, inst._playerlink) then
        unlink(inst)
        updatestate(inst)
        if inst.components.inventoryitem.owner == nil then
            -- startdecay(inst)
        end
    end
end

AddPrefabPostInit("abigail_flower",function(inst)
	-- 修改阿比盖尔之花
	inst:RemoveTag("irreplaceable")

    -- 取消默认的Task
    inst:CancelAllPendingTasks()
    inst:RemoveAllEventCallbacks()


	inst._chargestate = nil
    inst._playerlink = nil
    inst._container = nil
    inst._decaytask = nil
    inst._decaystart = nil

    inst._onremoveplayer = function()
        unlink(inst)
        updatestate(inst)
        if inst.components.inventoryitem.owner == nil then
            -- startdecay(inst)
        end
    end

    inst._onplayerkillthing = function(player, data)
        ondeath(inst, data.victim, player)
    end

    inst._onentitydeath = function(world, data)
        ondeath(inst, data.inst)
    end

    inst._ontogglecontainer = function(container)
        topocket(inst, container)
    end

    inst._onremovecontainer = function()
        unstore(inst)
    end



	inst.components.cooldown.onchargedfn = updatestate
    inst.components.cooldown.startchargingfn = updatestate
    inst.components.cooldown:StartCharging()

	--[[
    inst:WatchWorldState("phase", updatestate) 单机没有 WatchWorldState 方法
    inst:RemoveEventCallback("daytime", )
    inst:RemoveEventCallback("dusktime", )
    inst:RemoveEventCallback("nighttime",)
    --]]

    inst:ListenForEvent("daytime", function() updatestate(inst) end, GetWorld())
    inst:ListenForEvent("dusktime", function() updatestate(inst) end, GetWorld())
    inst:ListenForEvent("nighttime", function() updatestate(inst) end, GetWorld())

    updatestate(inst)
    -- startdecay(inst)

	
    inst:ListenForEvent("onputininventory", topocket)
    inst:ListenForEvent("ondropped", toground)
    

    inst.OnLoad = onload
    inst.OnSave = onsave
    inst.OnRemoveEntity = unlink
    inst.Refresh = refresh

    -- -- 将这多阿比盖尔之花添加到阿比盖尔之花的列表中
    -- inst:DoTaskInTime(0,function()
    --     -- body
    --     if GetPlayer().m_abigail_flowers == nil then
    --         GetPlayer().m_abigail_flowers = {}
    --     end
    --     GetPlayer().m_abigail_flowers[#GetPlayer().m_abigail_flowers+1] = inst
    -- end)
end)

local LOOT = { "abigail_flower" }

function refreshcontainer(container)
    for i = 1, container:GetNumSlots() do
        local item = container:GetItemInSlot(i)
        if item ~= nil and item.prefab == "abigail_flower" then
            item:Refresh()
        end
    end
end

local function unlink(inst)
    inst._playerlink.abigail = nil
    local inv = inst._playerlink.components.inventory
    refreshcontainer(inv)
    local activeitem = inv:GetActiveItem()
    if activeitem ~= nil and activeitem.prefab == "abigail_flower" then
        activeitem:Refresh()
    end

    for k, v in pairs(inv.opencontainers) do
        refreshcontainer(k.components.container)
    end

    -- inst:DoTaskInTime(0,function()
    --     -- body
    -- end)
    -- for i,v in ipairs(GetPlayer().m_abigail_flowers) do
    --     if v ~= nil then
    --         v:Refresh()
    --     end
    -- end
end

local function linktoplayer(inst, player)
    -- print("Abigail的LinkToPlayer被调用")
    inst.components.lootdropper:SetLoot(LOOT)
    -- inst.persists = false
    inst._playerlink = player
    player.abigail = inst
    -- player.components.leader:AddFollower(inst)
    if(player.abigail_flowers~=nil) then
        for k, v in pairs(player.abigail_flowers) do
            k:Refresh()
        end
    end
    player:ListenForEvent("onremove", unlink, inst)
end


AddPrefabPostInit("abigail",function(inst)
	-- 修改阿比盖尔
	inst.LinkToPlayer = linktoplayer
 --    GetPlayer().abigail = inst
    -- inst:DoTaskInTime(0,function(inst)
    --     if inst._playerlink == nil then
    --         inst:Remove()
    --     end
    --     -- body
    -- end)

    inst.OnLoad = function(inst, data)
        inst:LinkToPlayer(GetPlayer())
    end
end)

local function OnDespawn(inst)
    if inst.abigail ~= nil then
        inst.abigail.components.lootdropper:SetLoot(nil)
        inst.abigail.components.health:SetInvincible(true)
        inst.abigail:PushEvent("death")
        --in case the state graph got interrupted somehow, just force
        --removal after the dissipate animation should've finished
        inst.abigail:DoTaskInTime(25 * FRAMES, inst.abigail.Remove)
    end
end

local function OnSave(inst, data)
    if inst.abigail ~= nil then
        data.abigail = inst.abigail:GetSaveRecord()

    end
end

local function OnLoad(inst, data)
    if data ~= nil and data.abigail ~= nil and inst.abigail == nil then
        local abigail = SpawnSaveRecord(data.abigail)
        if abigail ~= nil then
            if inst.migrationpets ~= nil then
                table.insert(inst.migrationpets, abigail)
            end
            abigail.SoundEmitter:PlaySound("dontstarve/common/ghost_spawn")
            abigail:LinkToPlayer(inst)
        end
    end
end


AddPrefabPostInit("wendy", function(inst)
    -- 修改温蒂

    local abigail_flower_recipe = Recipe("abigail_flower", {Ingredient("petals", 6), Ingredient("nightmarefuel", 1)}, GLOBAL.RECIPETABS.MAGIC, GLOBAL.TECH.NONE )
    abigail_flower_recipe.sortkey = 1
    

    -- inst.OnDespawn = OnDespawn
    -- inst.OnSave = OnSave
    -- inst.OnLoad = OnLoad
end)