local Widget = require "widgets/widget"
local Text = require "widgets/text"
local Button = require "widgets/button"
local Image = require "widgets/image"

local ImageButtonHoldable = Class(Button, function(self, atlas, normal, focus, disabled)
    Button._ctor(self, "ImageButton")

    if not atlas then
        atlas = atlas or "images/ui.xml"
        normal = normal or "button.tex"
        focus = focus or "button_over.tex"
        disabled = disabled or "button_disabled.tex"
    end

    self.image = self:AddChild(Image())
    self.image:MoveToBack()

    self.atlas = atlas
	self.image_normal = normal
    self.image_focus = focus or normal
    self.image_disabled = disabled or normal
    
    self.image:SetTexture(self.atlas, self.image_normal)
	
	self.hold = false
end)

function ImageButtonHoldable:OnControl(control, down)
	
	if not self:IsEnabled() or not self.focus then return end
	
	if control == CONTROL_ACCEPT then
		if down then
			--TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_move")
			self.o_pos = self:GetLocalPosition()
			self:SetPosition(self.o_pos + self.clickoffset)
			self.down = true
		else
			if self.down then
				self.down = false
				self:SetPosition(self.o_pos)
				if self.onclick then
					self.hold = not self.hold
					self.onclick(self.hold)
				end
			end
		end
		
		return true
	end

end

function ImageButtonHoldable:OnGainFocus()
	ImageButtonHoldable._base.OnGainFocus(self)
    if self:IsEnabled() then
    	self.image:SetTexture(self.atlas, self.image_focus)
	end

    if self.image_focus == self.image_normal then
        self.image:SetScale(1.2,1.2,1.2)
    end

end

function ImageButtonHoldable:OnLoseFocus()
	ImageButtonHoldable._base.OnLoseFocus(self)
    if self:IsEnabled() and not self.hold then
    	self.image:SetTexture(self.atlas, self.image_normal)
	end

    if self.image_focus == self.image_normal then
        self.image:SetScale(1,1,1)
    end
end


function ImageButtonHoldable:Enable()
	ImageButtonHoldable._base.Enable(self)
    self.image:SetTexture(self.atlas, self.focus and self.image_focus or self.image_normal)

    if self.image_focus == self.image_normal then
        if self.focus then 
            self.image:SetScale(1.2,1.2,1.2)
        else
            self.image:SetScale(1,1,1)
        end
    end

end

function ImageButtonHoldable:Disable()
	ImageButtonHoldable._base.Disable(self)
	self.image:SetTexture(self.atlas, self.image_disabled)
end

function ImageButtonHoldable:GetSize()
    return self.image:GetSize()
end

return ImageButtonHoldable